@@ -6,7 +6,6 @@
  * inotify cron system
  *
  * Copyright (C) 2006, 2007, 2008, 2012 Lukas Jelinek, <lukas@aiken.cz>
- * Copyright (C) 2014, 2015 Andreas Altair Redmer, <altair.ibn.la.ahad.sy@gmail.com>
  *
  * This program is free software; you can use it, redistribute
  * it and/or modify it under the terms of the GNU General Public
@@ -70,7 +69,7 @@
           "These options may be used:\n" \
           "  -u <USER>, --user=<USER>     overrides current user (requires root privileges)\n" \
           "  -f <FILE>, --config=<FILE>   overrides default configuration file  (requires root privileges)\n\n" \
-          "For reporting bugs please use https://github.com/ar-/incron/issues\n"
+          "For reporting bugs please use http://bts.aiken.cz\n"
 
 
 
@@ -472,20 +471,20 @@ int main(int argc, char** argv)
   struct passwd* ppwd = NULL;
 
   if (chuser) {
-	if ((ppwd = getpwnam(user.c_str())) != NULL) {
+    if ((ppwd = getpwnam(user.c_str())) != NULL) {
       if (    setenv("LOGNAME",   ppwd->pw_name,   1) != 0
-		  ||  setenv("USER",      ppwd->pw_name,   1) != 0
-		  ||  setenv("USERNAME",  ppwd->pw_name,   1) != 0
-		  ||  setenv("HOME",      ppwd->pw_dir,    1) != 0
-		  ||  setenv("SHELL",     ppwd->pw_shell,  1) != 0)
+          ||  setenv("USER",      ppwd->pw_name,   1) != 0
+          ||  setenv("USERNAME",  ppwd->pw_name,   1) != 0
+          ||  setenv("HOME",      ppwd->pw_dir,    1) != 0
+          ||  setenv("SHELL",     ppwd->pw_shell,  1) != 0)
       {
-		perror("cannot set environment variables");
-		return 1;
-	  }
-	} else {
-	  fprintf(stderr, "user '%s' not found\n", user.c_str());
-	  return 1;
-	}
+        perror("cannot set environment variables");
+        return 1;
+      }
+    } else {
+      fprintf(stderr, "user '%s' not found\n", user.c_str());
+      return 1;
+    }
   } else {
     ppwd = getpwuid(uid);
     if (ppwd == NULL) {
