@@ -6,7 +6,6 @@
  * inotify cron system
  *
  * Copyright (C) 2006, 2007, 2008, 2012 Lukas Jelinek, <lukas@aiken.cz>
- * Copyright (C) 2014, 2015 Andreas Altair Redmer, <altair.ibn.la.ahad.sy@gmail.com>
  *
  * This program is free software; you can use it, redistribute
  * it and/or modify it under the terms of the GNU General Public
@@ -23,19 +22,19 @@
 #include <pwd.h>
 #include <syslog.h>
 #include <errno.h>
+#include <dirent.h>
 #include <sys/wait.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include <grp.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <cstdio>
 #include <cstring>
-#include <algorithm>
 
 #include "usertable.h"
 #include "incroncfg.h"
 #include "incrontab.h"
-#include "executor.h"
 
 #ifdef IN_DONT_FOLLOW
 #define DONT_FOLLOW(mask) InotifyEvent::IsType(mask, IN_DONT_FOLLOW)
@@ -53,12 +52,10 @@ extern volatile bool g_fFinish;
 extern SUT_MAP g_ut;
 
 
-#ifdef LOOPER
 void on_proc_done(InotifyWatch* pW)
 {
   pW->SetEnabled(true);
 }
-#endif
 
 
 EventDispatcher::EventDispatcher(int iPipeFd, Inotify* pIn, InotifyWatch* pSys, InotifyWatch* pUser)
@@ -182,12 +179,6 @@ void EventDispatcher::ProcessMgmtEvents()
         g_fFinish = true;
       }
       else if (!e.GetName().empty()) {
-		//ignore all names that start with a dot
-		const std::string eventName = e.GetName();
-		if (eventName[0] == '.') 
-			continue;
-		  
-		//start the actual process here
         SUT_MAP::iterator it = g_ut.find(IncronCfg::BuildPath(m_pSys->GetPath(), e.GetName()));
         if (it != g_ut.end()) {
           UserTable* pUt = (*it).second;
@@ -245,6 +236,8 @@ void EventDispatcher::ProcessMgmtEvents()
 
 
 
+size_t UserTable::m_maxForks = 0;
+bool UserTable::m_recursive = false;
 
 UserTable::UserTable(EventDispatcher* pEd, const std::string& rUser, bool fSysTable)
 : m_user(rUser),
@@ -261,6 +254,79 @@ UserTable::~UserTable()
   Dispose();
 }
 
+bool UserTable::IsDirectory(const std::string& sPath)
+{
+  static struct stat64 sStat;
+
+  if (-1 != lstat64(sPath.c_str(), &sStat) && S_ISDIR(sStat.st_mode)) {
+    return true;
+  }
+  else {
+    return false;
+  }
+}
+
+void UserTable::GetDirectoryTree(const std::string& sPath, TPathList& rResults)
+{
+  struct dirent* ent;
+  DIR* dir;
+
+  dir = opendir(sPath.c_str());
+  if (!dir) {
+    syslog(LOG_WARNING, "could not open directory %s: %s", sPath.c_str(), strerror(errno));
+    return;
+  }
+
+  rResults.push_back(sPath);
+
+  ent = readdir(dir);
+  while (ent) {
+    std::string sFullPath = sPath + "/" + ent->d_name;
+    std::string sEntry(ent->d_name);
+    if ("." != sEntry && ".." != sEntry && IsDirectory(sFullPath)) {
+      GetDirectoryTree(sFullPath, rResults);
+    }
+    ent = readdir(dir);
+  }
+  closedir(dir);
+}
+
+void UserTable::AddWatch(const std::string& sPath, IncronTabEntry* pEntry)
+{
+  InotifyWatch* pW = new InotifyWatch(sPath, pEntry->GetMask());
+
+  syslog(LOG_DEBUG, "adding directory watch %s", sPath.c_str());
+
+  // warning only - permissions may change later
+  if (!(m_fSysTable || MayAccess(sPath, DONT_FOLLOW(pEntry->GetMask()))))
+    syslog(LOG_WARNING, "access denied on %s - events will be discarded silently", sPath.c_str());
+
+  try {
+    m_in.Add(pW);
+    m_map.insert(IWCE_MAP::value_type(pW, pEntry));
+  } catch (InotifyException& e) {
+    if (m_fSysTable)
+      syslog(LOG_ERR, "cannot create watch for system table %s: (%i) %s", m_user.c_str(), e.GetErrorNumber(), strerror(e.GetErrorNumber()));
+    else
+      syslog(LOG_ERR, "cannot create watch for user %s: (%i) %s", m_user.c_str(), e.GetErrorNumber(), strerror(e.GetErrorNumber()));
+    delete pW;
+  }
+}
+
+void UserTable::RemoveWatch(const std::string& sPath)
+{
+  IWCE_MAP::iterator it = m_map.begin();
+  for (; it != m_map.end(); ++it) {
+    if (it->first->GetPath() == sPath) {
+      syslog(LOG_DEBUG, "removing directory watch %s", sPath.c_str());
+      m_in.Remove(it->first);
+      delete it->first;
+      m_map.erase(it);
+      return;
+    }
+  }
+}
+
 void UserTable::Load()
 {
   m_tab.Load(m_fSysTable
@@ -268,76 +334,23 @@ void UserTable::Load()
       : IncronTab::GetUserTablePath(m_user));
 
   int cnt = m_tab.GetCount();
-  
-  // add all subdirectories (recursively) as new tab entries with same events
   for (int i=0; i<cnt; i++) {
     IncronTabEntry& rE = m_tab.GetEntry(i);
-    
-    // skip if recursion is not wanted by user
-    if (rE.IsNoRecursion())
-		continue;
-    
-    std::vector<std::string> ssvec = Executor::getSubDirVec (rE.GetPath(),rE.IsDotDirs());
-	if (rE.GetPath().find("*") != std::string::npos) 
-	{
-		std::vector<std::string> allfilesvec = Executor::getAllFilesByDescriptor (rE.GetPath(),rE.IsDotDirs());
-		
-		// add to ssvec but without duplicates - in case user defined a dirctory with subdirectories via star descriptor
-		for (unsigned int j=0; j<allfilesvec.size(); j++) 
-		{
-			if(std::find(ssvec.begin(), ssvec.end(), allfilesvec[j]) == ssvec.end()) 
-			{
-				ssvec.push_back(allfilesvec[j]);
-			}
-		}
-	}    
-    for (unsigned int j=0; j<ssvec.size(); j++) {
-	  std::string subDir = ssvec[j];
-	  
-	  // skip if the subdir is the dir itself
-	  if (rE.GetPath() == subDir )
-		continue;
-		
-	  IncronTabEntry ite = IncronTabEntry(subDir, rE.GetMask(), rE.GetCmd());
-	  m_tab.Add(ite);
+
+    TPathList Paths;
+    if (m_recursive)
+      GetDirectoryTree(rE.GetPath(), Paths);
+    else
+      Paths.push_back(rE.GetPath());
+
+    for (TPathList::const_iterator iter = Paths.begin(); iter != Paths.end(); ++iter) {
+      AddWatch(*iter, &rE);
     }
   }
-  
-  // recount table elements too iterate over all of them
-  cnt = m_tab.GetCount();
-  for (int i=0; i<cnt; i++) {
-    IncronTabEntry& rE = m_tab.GetEntry(i);
-    // skip the wildcard selector, as they have been replace by the actual files
-	if (rE.GetPath().find("*") != std::string::npos) 
-		continue;
-	AddTabEntry(rE);
-  }
-  
+
   m_pEd->Register(this);
 }
 
-void UserTable::AddTabEntry(IncronTabEntry& rE)
-{
-    //syslog(LOG_INFO, "registering inotify for (%s)", rE.GetPath().c_str()); // TODO is this log spamming too much ?
-    InotifyWatch* pW = new InotifyWatch(rE.GetPath(), rE.GetMask());
-
-    // warning only - permissions may change later
-    if (!(m_fSysTable || MayAccess(rE.GetPath(), DONT_FOLLOW(rE.GetMask()))))
-      syslog(LOG_WARNING, "access denied on %s - events will be discarded silently", rE.GetPath().c_str());
-
-    try {
-      m_in.Add(pW);
-      m_map.insert(IWCE_MAP::value_type(pW, &rE));
-    } catch (InotifyException e) {
-      if (m_fSysTable)
-        syslog(LOG_ERR, "cannot create watch for system table %s: (%i) %s", m_user.c_str(), e.GetErrorNumber(), strerror(e.GetErrorNumber()));
-      else
-        syslog(LOG_ERR, "cannot create watch for user %s: (%i) %s", m_user.c_str(), e.GetErrorNumber(), strerror(e.GetErrorNumber()));
-      delete pW;
-    }	
-}
-
-
 void UserTable::Dispose()
 {
   m_pEd->Unregister(this);
@@ -366,6 +379,17 @@ void UserTable::Dispose()
   m_map.clear();
 }
 
+void UserTable::OnEventDirectory(const InotifyEvent& rEvt, InotifyWatch* pWatch, IncronTabEntry* pEntry)
+{
+  if (rEvt.IsType(IN_ISDIR)) {
+    std::string sDirectoryPath = pWatch->GetPath() + "/" + IncronTabEntry::GetSafePath(rEvt.GetName());
+
+    if (rEvt.IsType(IN_DELETE) || rEvt.IsType(IN_MOVED_FROM)) {
+      RemoveWatch(sDirectoryPath);
+    }
+  }
+}
+
 void UserTable::OnEvent(InotifyEvent& rEvt)
 {
   InotifyWatch* pW = rEvt.GetWatch();
@@ -378,34 +402,6 @@ void UserTable::OnEvent(InotifyEvent& rEvt)
   // discard event if user has no access rights to watch path
   if (!(m_fSysTable || MayAccess(pW->GetPath(), DONT_FOLLOW(rEvt.GetMask()))))
     return;
-    
-  //#if 0
-  // log output for each dir + file + event
-  std::string events;
-  rEvt.DumpTypes(events);
-  syslog(LOG_INFO, "PATH (%s) FILE (%s) EVENT (%s)", pW->GetPath().c_str() , IncronTabEntry::GetSafePath(rEvt.GetName()).c_str() , events.c_str());
-  //#endif
-  
-  // add new watch for newly created subdirs
-  if ( rEvt.IsType(IN_ISDIR) && (rEvt.IsType(IN_CREATE) || rEvt.IsType(IN_MOVED_TO)) )
-  {
-	Dispose();
-	sleep (1);
-	Load();
-  // this is the fast way of registering new subsirs, but it 
-  // misses new sub-sub dirs if they are created too fast in a row
-  // eg by : mkdir -p /tmp/a/b/c/d/e
-  // so the reload is for now the better way to go
-  // the complete reload also happens if the incrontab file changes
-  /*
-  m_pEd->Unregister(this);
-	std::string * pECmd = new std::string(pE->GetCmd().c_str());
-	IncronTabEntry * newEntry = new IncronTabEntry(completeFile, pE->GetMask(),  *pECmd);
-	m_tab.Add(*newEntry);
-	AddTabEntry(*newEntry);
-  m_pEd->Register(this);
-  */ 
-  }
 
   std::string cmd;
   const std::string& cs = pE->GetCmd();
@@ -437,9 +433,7 @@ void UserTable::OnEvent(InotifyEvent& rEvt)
         }
         else if (cs[px] == '&') {     // numeric mask
           char* s;
-#pragma GCC diagnostic ignored "-Wunused-result"  
           asprintf(&s, "%u", (unsigned) rEvt.GetMask());
-#pragma GCC diagnostic warning "-Wunused-result"
           cmd.append(s);
           free(s);
           oldpos = pos + 2;
@@ -456,22 +450,34 @@ void UserTable::OnEvent(InotifyEvent& rEvt)
   }
   cmd.append(cs.substr(oldpos));
 
+  int argc;
+  char** argv;
+  if (!PrepareArgs(cmd, argc, argv)) {
+    syslog(LOG_ERR, "cannot prepare command arguments");
+    return;
+  }
+
   if (m_fSysTable)
     syslog(LOG_INFO, "(system::%s) CMD (%s)", m_user.c_str(), cmd.c_str());
   else
     syslog(LOG_INFO, "(%s) CMD (%s)", m_user.c_str(), cmd.c_str());
-    
-#ifdef LOOPER
+
   if (pE->IsNoLoop())
     pW->SetEnabled(false);
-#endif
+
+  if (m_maxForks > 0 && s_procMap.size() >= m_maxForks) {
+    // time to wait for a forked process to finish before we continue
+    int status;
+    pid_t res = wait(&status);
+    UserTable::ClearProcess(res);
+  }
 
   pid_t pid = fork();
   if (pid == 0) {
 
     // for system table
     if (m_fSysTable) {
-      if (system(cmd.c_str()) != 0) // exec failed
+      if (execvp(argv[0], argv) != 0) // exec failed
       {
         syslog(LOG_ERR, "cannot exec process: %s", strerror(errno));
         _exit(1);
@@ -479,15 +485,10 @@ void UserTable::OnEvent(InotifyEvent& rEvt)
     }
     else {
       // for user table
-      RunAsUser(cmd);
-#ifdef LOOPER
-	  if (pE->IsNoLoop())
-		pW->SetEnabled(true);
-#endif
+      RunAsUser(argv);
     }
   }
   else if (pid > 0) {
-#ifdef LOOPER
     ProcData_t pd;
     if (pE->IsNoLoop()) {
       pd.onDone = on_proc_done;
@@ -499,17 +500,18 @@ void UserTable::OnEvent(InotifyEvent& rEvt)
     }
 
     s_procMap.insert(PROC_MAP::value_type(pid, pd));
-#endif
+
+    // Remove watches if it's a directory
+    OnEventDirectory(rEvt, pW, pE);
   }
   else {
-#ifdef LOOPER
     if (pE->IsNoLoop())
       pW->SetEnabled(true);
-#endif
 
     syslog(LOG_ERR, "cannot fork process: %s", strerror(errno));
   }
 
+  CleanupArgs(argc, argv);
 }
 
 IncronTabEntry* UserTable::FindEntry(InotifyWatch* pWatch)
@@ -521,6 +523,76 @@ IncronTabEntry* UserTable::FindEntry(InotifyWatch* pWatch)
   return (*it).second;
 }
 
+bool UserTable::PrepareArgs(const std::string& rCmd, int& argc, char**& argv)
+{
+  if (rCmd.empty())
+    return false;
+
+  StringTokenizer tok(rCmd, ' ', '\\');
+  std::deque<std::string> args;
+  while (tok.HasMoreTokens()) {
+    args.push_back(tok.GetNextToken());
+  }
+
+  if (args.empty())
+    return false;
+
+  argc = (int) args.size();
+  argv = new char*[argc+1];
+  argv[argc] = NULL;
+
+  for (int i=0; i<argc; i++) {
+    const std::string& s = args[i];
+    size_t len = s.length();
+    argv[i] = new char[len+1];
+    strcpy(argv[i], s.c_str());
+  }
+
+  return true;
+}
+
+void UserTable::CleanupArgs(int argc, char** argv)
+{
+  for (int i=0; i<argc; i++) {
+    delete[] argv[i];
+  }
+
+  delete[] argv;
+}
+
+void UserTable::FinishDone()
+{
+  pid_t res = 0;
+  int status = 0;
+  while ((res = waitpid(-1, &status, WNOHANG)) > 0) {
+    ClearProcess(res);
+  }
+}
+
+void UserTable::ClearProcess(pid_t pid)
+{
+  PROC_MAP::iterator it = s_procMap.find(pid);
+  if (it != s_procMap.end()) {
+    ProcData_t pd = (*it).second;
+    if (pd.onDone != NULL)
+      (*pd.onDone)(pd.pWatch);
+    s_procMap.erase(it);
+  }
+}
+
+void UserTable::ClearProcesses()
+{
+  int status = 0;
+
+  PROC_MAP::iterator it = s_procMap.begin();
+  while (it != s_procMap.end()) {
+    if (waitpid(it->first, &status, WNOHANG) > 0) {
+      ClearProcess(it->first);
+    }
+    it++;
+  }
+}
+
 bool UserTable::MayAccess(const std::string& rPath, bool fNoFollow) const
 {
   // first, retrieve file permissions
@@ -571,7 +643,7 @@ bool UserTable::MayAccess(const std::string& rPath, bool fNoFollow) const
   return false; // no access right found
 }
 
-void UserTable::RunAsUser(std::string cmd) const
+void UserTable::RunAsUser(char* const* argv) const
 {
   struct passwd* pwd = getpwnam(m_user.c_str());
   if (    pwd == NULL                 // check query result
@@ -596,8 +668,8 @@ void UserTable::RunAsUser(std::string cmd) const
       goto failed;
     }
   }
-  
-  execlp("/bin/bash","/bin/bash", "-c", cmd.c_str(), (char *)NULL);
+
+  execvp(argv[0], argv);  // this may return only on failure
 
 failed:
 
